unit eepromWrite;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    readBtn: TButton;
    writeBtn: TButton;
    StatusBar1: TStatusBar;
    readDataMemo: TMemo;
    procedure readBtnClick(Sender: TObject);
    procedure writeBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
AIOUSB;

{$R *.DFM}

function MotorolaDWord(W: LongWord): LongWord;
asm
  bswap EAX
end;

function IntAsStr(I: LongWord): String;
begin
  SetString(Result, PChar(@I), 4);
end;

function StrAsInt(S: String): LongWord;
begin
  case Length(S) of
    0: Result := 0;
    1: Result := PByte(@S[1])^;
    2: Result := PWord(@S[1])^;
    else Result := PLongWord(@S[1])^; //3 works because non-empty strings are null-terminated
  end;
end;

function StrAsSingle(S: AnsiString): Single;
begin
  if Length(S) < 4 then raise EParserError.Create('String too short to contain a floating-point Single.');
  Result := PSingle(@S[1])^;
end;

function StringReplaceC(const S: AnsiString; const OldChar, NewChar: AnsiChar): AnsiString;
var
  I: Integer;

begin
  Result := S;
  for I := 1 to Length(Result) do
    if Result[I] = OldChar then
      Result[I] := NewChar
  ;
end;

function StrToSpacedHex(Text: AnsiString): AnsiString;
const
  NybbleToHex: array[$0..$F] of AnsiChar = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex[Byte(Text[I]) shr  4];
    Result[I * 3 - 1] := NybbleToHex[Byte(Text[I]) and $F];
  end;
end;

function ValidateHexText(S: AnsiString): AnsiString;
var
  Content: TStringList;
  Buf: AnsiString;
  I, L: Integer;

begin
  S := StringReplaceC(S, ',', #13);
  S := StringReplaceC(S, ' ', #13);
  Content := TStringList.Create;
  try
    Content.Text := S;
    S := '';
    for I := Content.Count - 1 downto 0 do //Clear blank bytes and double delimiters
      if Length(Content[I]) = 0 then
        Content.Delete(I)
    ;
    L := Content.Count;
    SetString(Result, nil, L);
    for I := 0 to L - 1 do begin
      Buf := Content[I];
      Result[I + 1] := AnsiChar(StrToInt('$' + Buf));    // can throw exception
    end;
  finally
    Content.Free;
  end;
end;

function StrToFormattedHexEditStyle(Address: Integer; Text: String): String;
const
  NybbleToHex: array[$0..$F] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');
var
  I: Integer;
  aByte: String;
  Addr:String;
  startoffset :Integer;
begin
  aByte := '  ';
  Addr := IntToHex(Address and $FFF0, 4);
  Result := Addr + ': ';
  startoffset := Address and $000F;

  for I := 1 to startoffset do begin
    Result := Result + '.. ';
    if (I mod 8) = 0 then Result := Result + '   ';
  end;

  for I := 1 to Length(Text) do begin
    aByte[1] := NybbleToHex[Byte(Text[I]) shr  4];
    aByte[2] := NybbleToHex[Byte(Text[I]) and $F];
    Result := Result + aByte + ' ';

    if ((startoffset+i) mod 16 = 0) then
      Result := Result + #13#10 + IntToHex(Address + i,4) + ': '
    else
      if ((startoffset+i) mod 8 = 0) then
        Result := Result + '   ';
  end;
end;

procedure TForm1.readBtnClick(Sender: TObject);
var
  address : LongWord;
  Data: shortString;
  size : LongWord;
  Status: LongWord;

begin

    address := $1F00;
    size := $D6;
    SetString(Data, nil, size);

    Status := GenericVendorRead(diOnly, $A2,  address, 0, size, @Data[1]);

    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := ''
    else begin
            StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' reading data.';
            exit;
    end;

    // readDataMemo.Lines.Text := StrToSpacedHex(Data);
    readDataMemo.Lines.Text := StrToFormattedHexEditStyle(address, Data);
end;

procedure TForm1.writeBtnClick(Sender: TObject);
var
  Address: LongWord;
  Data: AnsiString;
  Status: LongWord;

begin
    address := $1F00;
    Data := ValidateHexText('D5 0A 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00');
    Status := GenericVendorWrite(diOnly, $A2, address, 0, Length(Data), @Data[1]);

    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := 'Data written to EEPROM: ' + StrToSpacedHex(Data)
    else
        StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' writing data.'
    ;

    address := $1F40;
    Data := ValidateHexText('00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 30 00 00 00 30 00 00 00 70 00 00 00 00 00 00 00 30 00 00 00 70 00 00 00 00 00 00 00 30 00 00 00 70 00 00 00 00 00');
    Status := GenericVendorWrite(diOnly, $A2, address, 0, Length(Data), @Data[1]);

    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := 'Data written to EEPROM: ' + StrToSpacedHex(Data)
    else
        StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' writing data.'
    ;

    address := $1F80;
    Data := ValidateHexText('00 00 30 00 00 00 70 00 00 00 00 00 00 00 30 00 00 00 70 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00');
    Status := GenericVendorWrite(diOnly, $A2, address, 0, Length(Data), @Data[1]);

    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := 'Data written to EEPROM: ' + StrToSpacedHex(Data)
    else
        StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' writing data.'
    ;

    address := $1FC0;
    Data := ValidateHexText('00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00');
    Status := GenericVendorWrite(diOnly, $A2, address, 0, Length(Data), @Data[1]);

    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := 'Data written to EEPROM: ' + StrToSpacedHex(Data)
    else
        StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' writing data.'
    ;
    readBtn.Click;
    readDataMemo.Lines.Add('done');
end;

end.
